from typing import List, Callable, Optional, Union

from ..common.datetime_range import DateTimeRange
from ..common import make_utc_datetime
import diskcache as dc
from ..common.variable import SpwcVariable
from ..common.variable import merge as merge_variables
from datetime import datetime, timedelta, timezone
from .version import str_to_version, version_to_str, Version
from ..config import cache_size
from functools import wraps

cache_version = str_to_version("1.1")


class CacheItem:
    def __init__(self, data, version):
        self.data = data
        self.version = version


class Cache:
    __slots__ = ['cache_file', '_data']

    def __init__(self, cache_path: str = ""):
        self._data = dc.FanoutCache(cache_path, shards=8, size_limit=int(float(cache_size.get())))
        if self.version < cache_version:
            self._data.clear()
            self.version = cache_version

    @property
    def version(self):
        return str_to_version(self._data.get("cache/version", default="0.0.0"))

    @version.setter
    def version(self, v: Union[str, Version]):
        self._data["cache/version"] = v if type(v) is str else version_to_str(v)

    def disk_size(self):
        return self._data.volume()

    def __len__(self):
        return len(self._data)

    def __del__(self):
        pass

    def keys(self):
        return [item for item in self._data]

    def __contains__(self, item):
        return item in self._data

    def __getitem__(self, key):
        return self._data[key]

    def __setitem__(self, key, value):
        self._data[key] = value

    def set(self, key, value, expire=None):
        self._data.set(key, value, expire=expire)






#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `spwc` package."""
import unittest
from datetime import datetime, timezone
import spwc
from ddt import ddt, data


class AMDARequest(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_get_variable(self):
        start_date = datetime(2006, 1, 8, 1, 0, 0, tzinfo=timezone.utc)
        stop_date = datetime(2006, 1, 8, 1, 0, 10, tzinfo=timezone.utc)
        result = spwc.get_data("amda/c1_b_gsm", start_date, stop_date, disable_proxy=True, disable_cache=True)
        self.assertIsNotNone(result)
        start_date = datetime(2016, 1, 8, 1, 0, 0, tzinfo=timezone.utc)
        stop_date = datetime(2016, 1, 8, 1, 0, 10, tzinfo=timezone.utc)
        parameter_id = "c1_hia_prest"
        result = spwc.get_data("amda/c1_hia_prest", start_date, stop_date, disable_proxy=True, disable_cache=True)
        self.assertIsNotNone(result)


@ddt
class CDARequest(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    @data(
        {
            "dataset": "MMS2_SCM_SRVY_L2_SCSRVY",
            "variable": "mms2_scm_acb_gse_scsrvy_srvy_l2",
            "start_time": datetime(2016, 6, 1, tzinfo=timezone.utc),
            "stop_time": datetime(2016, 6, 1, 0, 10, tzinfo=timezone.utc)
        },
        {
            "dataset": "THA_L2_FGM",
            "variable": "tha_fgl_gsm",
            "start_time": datetime(2014, 6, 1, tzinfo=timezone.utc),
            "stop_time": datetime(2014, 6, 1, 1, 10, tzinfo=timezone.utc)
        }
    )
    def test_get_variable(self, kw):
        result = spwc.get_data(f'cdaweb/{kw["dataset"]}/{kw["variable"]}', kw["start_time"], kw["stop_time"],
                               disable_proxy=True, disable_cache=True)
        self.assertIsNotNone(result)
